/**************************************************************
***************************************************************
wmudflib.c
These UDF's are designed to make using BLOBs in interbase easier.
They all use the "RETURNS PARAMETER x" option for strings
and blobs so Interbase is responsible for allocating the memory
for the return values. This seems to be the best option for thread
safe memory management.  This means that the UDF's have no knowledge
about how big the CSTRINGs to be returned can be, therefore care must 
be taken so that return values never exceed the returning CSTRING 
parameters maximum size.
***************************************************************
**************************************************************/



/**************************************************************
BLOB_TO_STRING
Converts a blob to its string representation. Ensure the CSTRING
is declared big enough for the returned string.
Parameters: SOURCEBLOB the blob to be returned as a string.
            CSTRING    dummy param to allow Interbase to perform 
                       memory management.
Returns:    CSTRING 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_TO_STRING
  BLOB, 
  CSTRING(32000)
  RETURNS PARAMETER 2
  ENTRY_POINT 'fn_blob_string' MODULE_NAME 'wmudflib';

DECLARE EXTERNAL FUNCTION BLOB_LENGTH
  BLOB
  RETURNS INTEGER BY VALUE
  ENTRY_POINT 'fn_blob_length' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_SUBSTR
Converts a blob to its string representation and returns a substring
starting at startpos and ending with endpos. Ensure the CSTRING
is declared big enough for the returned string.
Parameters: SOURCEBLOB the blob to be returned as a string.
            STARTPOS   the starting position of the substring (Starting at 1).
            ENDPOS     the end position of the substring (Starting at 1).
            CSTRING    dummy param to allow Interbase to perform 
                       memory management.
Returns:    CSTRING 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_SUBSTR
  BLOB, 
  INTEGER, 
  INTEGER, 
  CSTRING(32000)
  RETURNS PARAMETER 4
  ENTRY_POINT 'fn_blob_substr' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_LEFT
Converts a blob to its string representation and returns the 
leftmost characters. Ensure the CSTRING is declared big enough 
for the returned string.
Parameters: SOURCEBLOB the blob to be returned as a string.
            CHARACTERS the Number of charaters to be returned.
            CSTRING    dummy param to allow Interbase to perform 
                       memory management.
Returns:    CSTRING 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_LEFT
  BLOB, 
  INTEGER, 
  CSTRING(32000)
  RETURNS PARAMETER 3
  ENTRY_POINT 'fn_blob_left' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_RIGHT
Converts a blob to its string representation and returns the 
rightmost characters. Ensure the CSTRING is declared big enough 
for the returned string.
Parameters: SOURCEBLOB the blob to be returned as a string.
            CHARACTERS the Number of charaters to be returned.
            CSTRING    dummy param to allow Interbase to perform 
                       memory management.
Returns:    CSTRING 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_RIGHT
  BLOB, 
  INTEGER, 
  CSTRING(32000)
  RETURNS PARAMETER 3
  ENTRY_POINT 'fn_blob_right' MODULE_NAME 'wmudflib'; 

/**************************************************************
STRING_TO_BLOB
Converts a string to a blob.
Parameters: SOURCESTRING the string to be returned as a blob.
            BLOB         dummy param to allow Interbase to perform 
                         memory management.
Returns:    BLOB 
**************************************************************/

DECLARE EXTERNAL FUNCTION STRING_TO_BLOB
  CSTRING(32000), 
  BLOB
  RETURNS PARAMETER 2
  ENTRY_POINT 'fn_string_blob' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_APPEND_STRING
Appends the given string to the end of the blob.
Parameters: SOURCEBLOB the blob which is to be appended to.
            STRING     The string to be appended.
            BLOB       dummy param to allow Interbase to perform 
                       memory management.
Returns:    BLOB 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_APPEND_STRING
  BLOB, 
  CSTRING(32000), 
  BLOB
  RETURNS PARAMETER 3
  ENTRY_POINT 'fn_blob_string_append' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_INSERT_STRING
Inserts the given string at the given position in the blob.
Parameters: SOURCEBLOB the source blob.
            STRING     The string to be inserted.
            POSITION   The position to insert the string, starting at 1.
            BLOB       dummy param to allow Interbase to perform 
                       memory management.
Returns:    BLOB 
**************************************************************/
DECLARE EXTERNAL FUNCTION BLOB_INSERT_STRING
  BLOB, 
  CSTRING(32000), 
  INTEGER, 
  BLOB
  RETURNS PARAMETER 4
  ENTRY_POINT 'fn_blob_string_insert' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_COMPARE
Compares two blobs (Binary comparison) and returns 0 if identical,
otherwise returns the index of the first differing character.
Parameters: BLOBA      The first blob.
            BLOBB      The second blob
Returns:    POSITION 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_COMPARE
  BLOB, 
  BLOB
  RETURNS INTEGER BY VALUE
  ENTRY_POINT 'fn_blob_compare' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_DELETE_CHARS
Removes characters from within a blob, starting at startpos and 
ending with endpos.
Parameters: SOURCEBLOB the blob from which characters are to be 
                       removed.
            STARTPOS   the first character to remove (Starting at 1)
            ENDPOS     the last character to remove (Starting at 1)
            BLOB       dummy param to allow Interbase to perform 
                       memory management.
Returns:    BLOB 
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_DELETE_CHARS
  BLOB,
  INTEGER,
  INTEGER,
  BLOB
  RETURNS PARAMETER 4
  ENTRY_POINT 'fn_blob_delete_chars' MODULE_NAME 'wmudflib';

/**************************************************************
BLOB_STRING_POS
Returns the index of the first matching character from the search
string in the blob, or 0 if it is not found.
Parameters: SOURCEBLOB the blob to be searched.
            SEARCHSTR  The sting to search for.
Returns:    POSITION
**************************************************************/

DECLARE EXTERNAL FUNCTION BLOB_STRING_POS
  BLOB,
  CSTRING(32000)
  RETURNS INTEGER BY VALUE
  ENTRY_POINT 'fn_blob_string_pos' MODULE_NAME 'wmudflib';